import os
import json

from fastapi import APIRouter
from pydantic import BaseModel
import azure.cosmos.exceptions as exceptions
from ..database.mycosmosdb import callConnect
import stripe
import datetime
import logging


NODE_DEV = os.getenv("NODE_DEV")


if NODE_DEV == "production":
    MY_DOMAIN = "https://metricplatformapi.com/"
    MY_HOME = "https://www.metric-platform.com/"
elif NODE_DEV == "test":
    MY_DOMAIN = "https://metricplatform-staging.azurewebsites.net/"
    MY_HOME = "https://metricplatform-info-metric-esgc-metric-dev.vercel.app/"
else:
    MY_DOMAIN = "http://localhost:8000/"
    MY_HOME = "http://localhost:3000/"


class Newuser22(BaseModel):
    oid: str
    city: str
    email: str
    session_id: str
    subs: int


class Newuser(BaseModel):
    oid: str
    city: str
    email: str
    session_id: str


class Passport2(BaseModel):
    oid: str
    city: str
    email: str


class Subs(BaseModel):
    oid: str
    city: str
    fund: str
    subs: int


def read_item(container, doc_id, account_number):
    response = container.read_item(item=doc_id, partition_key=account_number)
    return response


def create_item(container, doc):
    response = container.create_item(body=doc)
    return response


subroute = APIRouter()

current_dir_location = os.path.dirname(os.path.abspath(__file__))
email_eip_path = os.path.join(current_dir_location, "..", "static", "eip_email.json")

with open(email_eip_path) as user_emails:
    eip_email_to_check = json.load(user_emails)

# Frog emails to bypass the subscription check
allowed_emails = [
    "Beth.Adamson@frogcapital.com",
    "m.seebeck@caspar-health.com",
    "andre@genasystech.com",
    "Kirsty.jordan@genasystech.com",
    "Christelle@genasystech.com",
    "Sara.Holmberg@winningtemp.com",
    "josefin.Lindell@winningtemp.com",
    "Nick.harber@clue.co.uk",
    "Jenny.thomson@clue.co.uk",
    "driss.khouja@housfy.com",
    "sophie@oxfordmedicalsimulation.com",
    "john.peterson@flexe.com",
    "tony@homee.com",
    "yasmin.zarabi@parsable.com",
    "brian.pope@passportinc.com",
    "mgroeninger@voltus.co",
    "rick@klearnow.com",
    "eddie.chen@fictiv.com",
    "mragazzo@optimaldynamics.com",
    "charles.constanti@swimlane.com",
    "doug@voltus.co",
    "mculbertson@omnidian.com",
    "laurie@elementanalytics.com",
    "mmurday@metric-esg.com",
]

"""
Endpoint that checks if your subscription is valid. If you have a Session_ID we check with stripe to see if you have paid your subscription or not. If you have you are redirected to the dashboard, if you have not you are redirected to billing. If you do not have a record in the database then you are redirected to the signup page.
"""


@subroute.post("/subcheck")
def subscription_check(passport: Passport2):
    try:
        db_result = read_item(callConnect.container, passport.oid, passport.city)

    except exceptions.CosmosHttpResponseError as e:
        try:
            db_result2 = read_item(
                callConnect.container5, passport.email, passport.email
            )

        except exceptions.CosmosHttpResponseError as e:
            logging.error("This is an error message for /subcheck container5")
            return {"url": "/signup", "atype": True}

        if "team" in db_result2:
            if db_result2["session_id"] == "none":
                return {
                    "url": "/dashboard/home",
                    "atype": True,
                    "ptype": db_result2["session_id"],
                    "plan": db_result2["plan_id"],
                    "oid": db_result2["oid"],
                    "city": db_result2["city"],
                    "email": db_result2["email"],
                }

            checkout_session = stripe.checkout.Session.retrieve(
                db_result2["session_id"]
            )
            paid = stripe.Subscription.list(limit=3, customer=checkout_session.customer)

            is_allowed = checkout_session["customer_details"]["email"] in allowed_emails
            if is_allowed:
                return {
                    "url": "/dashboard/home",
                    "atype": True,
                    "ptype": db_result2["session_id"],
                    "plan": db_result2["plan_id"],
                    "oid": db_result2["oid"],
                    "city": db_result2["city"],
                    "email": db_result2["email"],
                }
            if "data" in paid and len(paid["data"]) == 0:
                return {
                    "url": "/billing",
                    "atype": False,
                    "ptype": db_result["session_id"],
                }
            else:
                paid2 = paid.data[0]["items"]["data"][0]["plan"]["active"]
                if paid2:
                    return {
                        "url": "/dashboard/home",
                        "atype": True,
                        "ptype": db_result2["session_id"],
                        "plan": db_result2["plan_id"],
                        "oid": db_result2["oid"],
                        "city": db_result2["city"],
                        "email": db_result2["email"],
                    }

                else:
                    return {
                        "url": "/billing",
                        "atype": False,
                        "ptype": db_result2["session_id"],
                    }

    if "session_id" in db_result:
        if db_result["session_id"] != "none":
            checkout_session = stripe.checkout.Session.retrieve(db_result["session_id"])
            is_allowed = checkout_session["customer_details"]["email"] in allowed_emails
            if is_allowed:
                return {
                    "url": "/dashboard/home",
                    "atype": True,
                    "ptype": db_result["session_id"],
                    "plan": db_result["plan_id"],
                }
            paid = stripe.Subscription.list(limit=3, customer=checkout_session.customer)
            if "data" in paid and len(paid["data"]) == 0:
                return {
                    "url": "/billing",
                    "atype": False,
                    "ptype": db_result["session_id"],
                }
            else:
                paid2 = paid.data[0]["items"]["data"][0]["plan"]["active"]
                if paid2:
                    return {
                        "url": "/dashboard/home",
                        "atype": True,
                        "ptype": db_result["session_id"],
                        "plan": db_result["plan_id"],
                    }

                else:
                    return {
                        "url": "/billing",
                        "atype": False,
                        "ptype": db_result["session_id"],
                    }
        else:
            return {
                "url": "/dashboard/home",
                "atype": True,
                "ptype": db_result["session_id"],
                "plan": db_result["plan_id"],
            }


"""
Endpoint used to register a user once they are returned from their Checkout Session from Stripe
"""


@subroute.post("/register")
async def register_user(newuser: Newuser22):
    checkout_session = stripe.checkout.Session.retrieve(newuser.session_id)
    plan_id = stripe.Subscription.list(
        limit=3, customer=checkout_session.customer
    ).data[0]["items"]["data"][0]["plan"]["id"]
    nowZ = datetime.datetime.now()
    yearZ = nowZ.strftime("%Y")
    monthZ = nowZ.strftime("%m")
    dayZ = nowZ.strftime("%d")
    hourZ = nowZ.strftime("%H")
    minuteZ = nowZ.strftime("%M")
    all_timeZ = yearZ + "-" + monthZ + "-" + dayZ + "T" + hourZ + ":" + minuteZ
    all_timeZ2 = yearZ + "-" + monthZ + "-" + dayZ

    email = newuser.email

    if email in eip_email_to_check:
        email = newuser.oid

    user2 = {
        "id": newuser.oid,
        "city": newuser.city,
        "email": email,
        "session_id": newuser.session_id,
        "sub_created": all_timeZ,
        "plan_id": plan_id,
    }

    userx = {"id": newuser.oid, "email": email, "carbonmeasure": []}
    userx2 = {"id": newuser.oid, "email": email, "diversitymeasure": []}
    userx3 = {"id": newuser.oid, "email": email, "profile": []}
    callConnect.container11.upsert_item(body=userx3)
    userx4 = {
        "id": newuser.oid,
        "email": email,
        "improvement": [
            {
                "label": "IS",
                "metrics": [
                    {
                        "datestr": all_timeZ2,
                        "funding": "Pre-Seed & Seed",
                        "emissionsIntensity": False,
                        "emissionsIntensityFin": False,
                        "measureBaseline": False,
                        "measureBaselineFin": False,
                        "pickOffice": False,
                        "pickOfficeFin": False,
                        "biasedLang": False,
                        "biasedLangFin": False,
                        "createBoard": False,
                        "createBoardFin": False,
                        "diverseBoard": False,
                        "diverseBoardFin": False,
                        "scope3": False,
                        "scope3Fin": False,
                        "neutralTarget": False,
                        "neutralTargetFin": False,
                        "physicalProducts": False,
                        "physicalProductsFin": False,
                        "suppliers": False,
                        "suppliersFin": False,
                        "officeSupplies": False,
                        "officeSuppliesFin": False,
                        "officeRecycle": False,
                        "officeRecycleFin": False,
                        "productImpact": False,
                        "productImpactFin": False,
                        "carbonEmployee": False,
                        "carbonEmployeeFin": False,
                        "energyPlan": False,
                        "energyPlanFin": False,
                        "waterPlan": False,
                        "waterPlanFin": False,
                        "wastePlan": False,
                        "wastePlanFin": False,
                        "candidateScreenings": False,
                        "candidateScreeningsFin": False,
                        "humanResources": False,
                        "humanResourcesFin": False,
                        "unconsciousBias": False,
                        "unconsciousBiasFin": False,
                        "compensationAudit": False,
                        "compensationAuditFin": False,
                        "equitableCompensation": False,
                        "equitableCompensationFin": False,
                        "diversityInclusion": False,
                        "diversityInclusionFin": False,
                        "recruitmentProcess": False,
                        "recruitmentProcessFin": False,
                        "biasEmployees": False,
                        "biasEmployeesFin": False,
                        "regularEmployee": False,
                        "regularEmployeeFin": False,
                        "anonymousHotline": False,
                        "anonymousHotlineFin": False,
                        "socialImpact": False,
                        "socialImpactFin": False,
                        "employeeAffinity": False,
                        "employeeAffinityFin": False,
                        "employeeWellbeing": False,
                        "employeeWellbeingFin": False,
                        "employeeCivic": False,
                        "employeeCivicFin": False,
                        "formalDiscrimination": False,
                        "formalDiscriminationFin": False,
                        "formalMental": False,
                        "formalMentalFin": False,
                        "privacyRegulation": False,
                        "privacyRegulationFin": False,
                        "addingIndependent": False,
                        "addingIndependentFin": False,
                        "publishDiversity": False,
                        "publishDiversityFin": False,
                        "regularEnterprise": False,
                        "regularEnterpriseFin": False,
                        "designateIndependent": False,
                        "designateIndependentFin": False,
                        "publicDisclosure": False,
                        "publicDisclosureFin": False,
                        "corporateSustainability": False,
                        "corporateSustainabilityFin": False,
                    }
                ],
            }
        ],
    }
    userx5 = {"id": email, "oid": newuser.oid, "team": []}
    callConnect.container13.upsert_item(body=userx5)
    userxCustom = {"id": newuser.oid, "email": email, "custommeasure": []}

    if (
        plan_id == "price_1KU9EIJnvGQfuwqX1pAkeLP0"
        or plan_id == "price_1MC79wJnvGQfuwqXeLIJXVws"
        or plan_id == "price_1MN2vNJnvGQfuwqX3ZT4NiyE"
    ):
        subss = {"subs": newuser.subs}
        user2.update(subss)
        callConnect.container15.upsert_item(body=userxCustom)

    if (
        plan_id == "price_1KUEG2JnvGQfuwqX5krVeoWW"
        or plan_id == "price_1MOTDPJnvGQfuwqXvpHrxCzr"
        or plan_id == "price_1LrjAwJnvGQfuwqXXIkFAbyl"
    ):
        callConnect.container7.upsert_item(body=userx)
        callConnect.container8.upsert_item(body=userx2)
        callConnect.container12.upsert_item(body=userx4)
        callConnect.container15.upsert_item(body=userxCustom)

    try:
        create_item(callConnect.container, user2)
        return {
            "url": "/dashboard/home",
            "atype": True,
            "ptype": newuser.session_id,
            "plan": plan_id,
        }

    except exceptions.CosmosHttpResponseError as e:
        logging.error("This is an error message for /register")
        return {"url": "/", "atype": False}


"""
Endpoint used to register a user once they are returned from their Checkout Session from Stripe
"""


@subroute.post("/registerSWAP")
async def register_user(newuser: Newuser):
    try:
        db_result = read_item(callConnect.container, newuser.oid, newuser.city)

    except exceptions.CosmosHttpResponseError as e:
        logging.error("This is an error message for /registerSWAP")
        return {"url": "/", "atype": False}

    checkout_session = stripe.checkout.Session.retrieve(newuser.session_id)
    plan_id = stripe.Subscription.list(
        limit=3, customer=checkout_session.customer
    ).data[0]["items"]["data"][0]["plan"]["id"]
    nowZ = datetime.datetime.now()
    yearZ = nowZ.strftime("%Y")
    monthZ = nowZ.strftime("%m")
    dayZ = nowZ.strftime("%d")
    hourZ = nowZ.strftime("%H")
    minuteZ = nowZ.strftime("%M")
    all_timeZ = yearZ + "-" + monthZ + "-" + dayZ + "T" + hourZ + ":" + minuteZ

    target = {
        "session_id": newuser.session_id,
        "sub_created": all_timeZ,
        "plan_id": plan_id,
    }
    db_result.update(target)

    try:
        callConnect.container.upsert_item(body=db_result)
        return {
            "url": f"{MY_HOME}",
            "atype": True,
            "ptype": newuser.session_id,
            "plan": plan_id,
        }

    except exceptions.CosmosHttpResponseError as e:
        logging.error("This is an error message for /registerSWAP")
        return {"url": "/", "atype": False}


"""
This is the registration for the free trial
"""


@subroute.post("/register2")
def register_user2(newuser: Newuser):
    si = "none"
    nowZ = datetime.datetime.now()
    yearZ = nowZ.strftime("%Y")
    monthZ = nowZ.strftime("%m")
    dayZ = nowZ.strftime("%d")
    hourZ = nowZ.strftime("%H")
    minuteZ = nowZ.strftime("%M")
    all_timeZ = yearZ + "-" + monthZ + "-" + dayZ + "T" + hourZ + ":" + minuteZ
    all_timeZ2 = yearZ + "-" + monthZ + "-" + dayZ

    user2 = {
        "id": newuser.oid,
        "city": newuser.city,
        "email": newuser.email,
        "session_id": si,
        "sub_created": all_timeZ,
        "plan_id": "free",
    }

    userx = {"id": newuser.oid, "email": newuser.email, "carbonmeasure": []}
    callConnect.container7.upsert_item(body=userx)
    userx2 = {"id": newuser.oid, "email": newuser.email, "diversitymeasure": []}
    callConnect.container8.upsert_item(body=userx2)
    userx3 = {"id": newuser.oid, "email": newuser.email, "profile": []}
    callConnect.container11.upsert_item(body=userx3)
    userx4 = {
        "id": newuser.oid,
        "email": newuser.email,
        "improvement": [
            {
                "label": "IS",
                "metrics": [
                    {
                        "datestr": all_timeZ2,
                        "funding": "Pre-Seed & Seed",
                        "emissionsIntensity": False,
                        "emissionsIntensityFin": False,
                        "measureBaseline": False,
                        "measureBaselineFin": False,
                        "pickOffice": False,
                        "pickOfficeFin": False,
                        "biasedLang": False,
                        "biasedLangFin": False,
                        "createBoard": False,
                        "createBoardFin": False,
                        "diverseBoard": False,
                        "diverseBoardFin": False,
                        "scope3": False,
                        "scope3Fin": False,
                        "neutralTarget": False,
                        "neutralTargetFin": False,
                        "physicalProducts": False,
                        "physicalProductsFin": False,
                        "suppliers": False,
                        "suppliersFin": False,
                        "officeSupplies": False,
                        "officeSuppliesFin": False,
                        "officeRecycle": False,
                        "officeRecycleFin": False,
                        "productImpact": False,
                        "productImpactFin": False,
                        "carbonEmployee": False,
                        "carbonEmployeeFin": False,
                        "energyPlan": False,
                        "energyPlanFin": False,
                        "waterPlan": False,
                        "waterPlanFin": False,
                        "wastePlan": False,
                        "wastePlanFin": False,
                        "candidateScreenings": False,
                        "candidateScreeningsFin": False,
                        "humanResources": False,
                        "humanResourcesFin": False,
                        "unconsciousBias": False,
                        "unconsciousBiasFin": False,
                        "compensationAudit": False,
                        "compensationAuditFin": False,
                        "equitableCompensation": False,
                        "equitableCompensationFin": False,
                        "diversityInclusion": False,
                        "diversityInclusionFin": False,
                        "recruitmentProcess": False,
                        "recruitmentProcessFin": False,
                        "biasEmployees": False,
                        "biasEmployeesFin": False,
                        "regularEmployee": False,
                        "regularEmployeeFin": False,
                        "anonymousHotline": False,
                        "anonymousHotlineFin": False,
                        "socialImpact": False,
                        "socialImpactFin": False,
                        "employeeAffinity": False,
                        "employeeAffinityFin": False,
                        "employeeWellbeing": False,
                        "employeeWellbeingFin": False,
                        "employeeCivic": False,
                        "employeeCivicFin": False,
                        "formalDiscrimination": False,
                        "formalDiscriminationFin": False,
                        "formalMental": False,
                        "formalMentalFin": False,
                        "privacyRegulation": False,
                        "privacyRegulationFin": False,
                        "addingIndependent": False,
                        "addingIndependentFin": False,
                        "publishDiversity": False,
                        "publishDiversityFin": False,
                        "regularEnterprise": False,
                        "regularEnterpriseFin": False,
                        "designateIndependent": False,
                        "designateIndependentFin": False,
                        "publicDisclosure": False,
                        "publicDisclosureFin": False,
                        "corporateSustainability": False,
                        "corporateSustainabilityFin": False,
                    }
                ],
            }
        ],
    }
    callConnect.container12.upsert_item(body=userx4)
    userx5 = {"id": newuser.email, "oid": newuser.oid, "team": []}
    callConnect.container13.upsert_item(body=userx5)
    userxCustom = {"id": newuser.oid, "email": newuser.email, "custommeasure": []}
    callConnect.container15.upsert_item(body=userxCustom)

    try:
        create_item(callConnect.container, user2)
        return {"url": "/dashboard/home", "atype": True, "ptype": True, "plan": "free"}

    except exceptions.CosmosHttpResponseError as e:
        logging.error("This is an error message for /register2")
        return {"url": "/", "atype": False}


"""
Endpoint used to register new company subscriptions.
We retrieve the users data then check to see if they already have subscriptions.
If they do then we append to that, if they do not then we create and add the  object.
"""


@subroute.post("/newsubs")
async def register_fund(sub: Subs):
    try:
        db_result = read_item(callConnect.container, sub.oid, sub.city)

    except exceptions.CosmosHttpResponseError as e:
        logging.error("This is an error message for /newsubs")
        return {"url": "/", "atype": False}

    if "subs" in db_result:
        user2 = {
            "id": db_result["id"],
            "city": db_result["city"],
            "email": db_result["email"],
            "session_id": db_result["session_id"],
            "sub_created": db_result["sub_created"],
            "plan_id": db_result["plan_id"],
            "funds": db_result["funds"],
            "subs": db_result["subs"],
        }
        subss = {"subs": db_result["subs"] + sub.subs}
    else:
        user2 = {
            "id": db_result["id"],
            "city": db_result["city"],
            "email": db_result["email"],
            "session_id": db_result["session_id"],
            "sub_created": db_result["sub_created"],
            "plan_id": db_result["plan_id"],
            "funds": db_result["funds"],
        }
        subss = {"subs": sub.subs}

    user2.update(subss)
    callConnect.container.upsert_item(body=user2)

    return {"url": f"{MY_HOME}dashboard/fundoverview/"}
